//+=============================================================================================+
//+ Project  : vmix16                                                                           +  
//+ Author   : Mike G7GTN                                                                       +
//+ Version  : 1.00                                                                             +
//+ IDE      : Compiled in Ardunio 1.8.0                                                        +
//+ Date     : July 2018                                                                        +
//+=============================================================================================+
//+
//+ Purpose: -
//+
//+             Send keyboard VMIX shortcuts from a 16 way keypad matrix
//+
//+ Libraries: -
//+
//+
//+==============================================================================================+

#define VERSION 0X100
#define KEY_LEFT_CTRL  0x80 
#define TEST		  // Comment out when have I2C Address

#include "Keyboard.h"
#include <Keypad_I2C.h>
#include <Keypad.h>
#include <Wire.h>
#define I2CADDR 0x38      // Change for your modules set PCF8574 I2C address
const byte ROWS = 4;      // Four rows
const byte COLS = 4;      // Three columns
char keys[ROWS][COLS] = {
  {'1','2','3','A'},
  {'4','5','6','B'},
  {'7','8','9','C'},
  {'*','0','#','D'}
};
// Matrix keypad, bit numbers of PCF8574 I/O port
byte rowPins[ROWS] = {0, 1, 2, 3};  // Connect to the Row pinouts of the keypad
byte colPins[COLS] = {4, 5, 6, 7};  // Connect to the Column pinouts of the keypad

Keypad_I2C kpd( makeKeymap(keys), rowPins, colPins, ROWS, COLS, I2CADDR, PCF8574 );

//-------------------------------------------------------------------------------------
//+ Hardware setup
//-------------------------------------------------------------------------------------
void setup() 
{
  Wire.begin( );
  kpd.begin( makeKeymap(keys) );                // Start keypad
  Keyboard.begin();
  Serial.begin(9600);
  Serial.flush();
  scanbus();                                    // Check I2C Address
}

//---------------------------------------------------------------------------------------
//+ I2C Scanner
//---------------------------------------------------------------------------------------
void scanbus()
{
  Serial.println ("[I2C Scanning]");
  byte count = 0;
  Wire.begin();
  for (byte i = 1; i < 127; i++)                  // I2C address range
  {
    Wire.beginTransmission (i);
    if (Wire.endTransmission () == 0)
      {
      Serial.print ("Found device at address: ");
      Serial.print ("Decimal ");
      Serial.print (i, DEC);
      Serial.print (" Hexadecimal(0x");
      Serial.print (i, HEX);
      Serial.println (")");
      count++;
      delay (1);  
      } 
  } 
  Serial.print ("Found ");
  Serial.print (count, DEC);
  Serial.println (" I2C device(s).");
  Serial.println ("[Finished Scan]");     
}  

//---------------------------------------------------------------------------------------
//+ Get Keypad presses
//+ Each key sends an associated Control - Shortcut key
//---------------------------------------------------------------------------------------
 void readkeypad()
 {
  char key = kpd.getKey(); 
  if (key) //  != NO_KEY)
    switch(key)
    {
    case '0':                             // Send Control - A
      //Serial.print("0"); 
      Keyboard.press(KEY_LEFT_CTRL); 
      Keyboard.write('A'); 
      break;  
      
    case '1':
      //Serial.print("1");  
      Keyboard.press(KEY_LEFT_CTRL); 
      Keyboard.write('B');
      break;
    
    case '2':
      //Serial.print("2");
      Keyboard.press(KEY_LEFT_CTRL); 
      Keyboard.write('C');
      break;

    case '3':
      //Serial.print("3");
      Keyboard.press(KEY_LEFT_CTRL); 
      Keyboard.write('D'); 
      break;

    case '4':
      //Serial.print("4"); 
      Keyboard.press(KEY_LEFT_CTRL); 
      Keyboard.write('E'); 
      break;
 
    case '5':
      //Serial.print("5");
      Keyboard.press(KEY_LEFT_CTRL); 
      Keyboard.write('F'); 
      break;

    case '6':
      //Serial.print("6"); 
      Keyboard.press(KEY_LEFT_CTRL); 
      Keyboard.write('G'); 
      break;

    case '7':
      //Serial.print("7"); 
      Keyboard.press(KEY_LEFT_CTRL); 
      Keyboard.write('H'); 
      break;

    case '8':
      //Serial.print("8");
      Keyboard.press(KEY_LEFT_CTRL); 
      Keyboard.write('I'); 
      break;

    case '9':
      //Serial.print("9");
      Keyboard.press(KEY_LEFT_CTRL); 
      Keyboard.write('J'); 
      break;

    case '*':
      //Serial.print("*");
      Keyboard.press(KEY_LEFT_CTRL); 
      Keyboard.write('K'); 
      break;
     
    case '#':
      //Serial.print("#");
      Keyboard.press(KEY_LEFT_CTRL); 
      Keyboard.write('L'); 
      break;

    case 'A':
      //Serial.print("A");
      Keyboard.press(KEY_LEFT_CTRL);
      Keyboard.write('M'); 
      break;

    case 'B':
      //Serial.print("B");
      Keyboard.press(KEY_LEFT_CTRL);
      Keyboard.write('N'); 
      break;

    case 'C':
      //Serial.print("C");
      Keyboard.press(KEY_LEFT_CTRL);
      Keyboard.write('O'); 
      break;
     
    case 'D':
      //Serial.print("D");
      Keyboard.press(KEY_LEFT_CTRL);      // Send Control - P
      Keyboard.write('P'); 
      break;     
  }  
}

//-------------------------------------------------------------------------------------
//+ Main Loop
//-------------------------------------------------------------------------------------
void loop() 
{
  readkeypad();
}
